@@ -50,21 +50,18 @@ class AppleUSBAudioEngine;
 class AppleUSBAudioDevice : public IOAudioDevice {
     OSDeclareDefaultStructors (AppleUSBAudioDevice);
 
-    USBAudioConfigObject *	usbAudio;
+public:
     IOUSBInterface *		controlInterface;
+
+protected:
+    USBAudioConfigObject *	usbAudio;
     IOUSBInterface *		previousInterface;
     IORecursiveLock *		interfaceLock;
-#if TIMERSTREAM
-    IONotifier *			deviceNotifier;
-#endif
     OSObject *				interfaceVendor;
     OSObject *				interfaceProduct;
     OSObject *				deviceReleaseNumber;
     OSObject *				configurationValue;
     OSObject *				interfaceNumber;
-#if TIMERSTREAM
-    bool					hasBeenUnattached;
-#endif
     
 public:
     virtual bool init (OSDictionary *properties);
@@ -74,32 +71,17 @@ public:
     virtual void performStop (IOService *provider);
 	virtual Boolean ShouldUpdatePRAM (void);
 	Boolean FindSoundNode (void);
-#if DOUBLEPARENT
 	virtual bool requestTerminate (IOService * provider, IOOptionBits options);
-#endif
     virtual bool terminate (IOOptionBits options = 0);
     virtual bool finalize(IOOptionBits options);
     virtual IOReturn message (UInt32 type, IOService * provider, void * arg);
     virtual IOReturn activateAudioEngine (IOAudioEngine *audioEngine, bool shouldStartAudioEngine, UInt8 interfaceNum, UInt8 altInterfaceNum);
 	static	IOReturn deviceRequest (IOUSBDevRequest * request, AppleUSBAudioDevice * self, IOUSBCompletion * completion = 0);
     virtual bool allStreamsStopped ();
-#if 0
-    virtual UInt32 getNumAttachedAudioDevices ();
-#endif
+	virtual IOReturn performPowerStateChange (IOAudioDevicePowerState oldPowerState, IOAudioDevicePowerState newPowerState, UInt32 *microSecsUntilComplete);
     virtual USBAudioConfigObject * GetUSBAudioConfigObject (void) {return usbAudio;}
     virtual void streamStopped (IOAudioEngine *audioEngine);
-	virtual IOReturn performPowerStateChange (IOAudioDevicePowerState oldPowerState, IOAudioDevicePowerState newPowerState, UInt32 *microSecsUntilComplete);
-#if TIMERSTREAM
-    virtual bool isUnattachedDeviceRegistered ();
-    virtual void registerUnattachedDevice ();
-    virtual void unregisterUnattachedDevice ();
-	static bool interfacePublished (AppleUSBAudioDevice *device, void *ref, IOService *newService);
-    virtual bool reinitWithInterface (IOUSBInterface *interface);
-    static IOReturn performUnattachedDeviceRegistrationAction (OSObject *owner, void *arg1, void *arg2, void *arg3, void *arg4);
-    virtual bool performUnattachedDeviceRegistration ();
-    static IOReturn reinitWithInterfaceAction (OSObject *owner, void *arg1, void *arg2, void *arg3, void *arg4);
-#endif
-    
+
 #ifdef DEBUG
     virtual void retain() const;
     virtual void release() const;
