@@ -70,7 +70,7 @@ void USBAudioConfigObject::DumpConfigMemoryToIOLog (void) {
 	while (*descriptorPtr != 0) {
 		length = *descriptorPtr;
 		for (descriptorIndex = 0; descriptorIndex < length; descriptorIndex++) {
-			IOLog ("%2x ", *descriptorPtr++);
+			debugIOLog ("%2x ", *descriptorPtr++);
 		}
 		debugIOLog ("\n");
 	}
@@ -293,7 +293,7 @@ UInt16 USBAudioConfigObject::GetFormat (UInt8 interfaceNum, UInt8 altInterfaceNu
 	formatTag = TYPE_I_UNDEFINED;
 	thisStream = GetStreamObject (interfaceNum, altInterfaceNum);
 	if (thisStream)
-		formatTag = thisStream->GetFormatTag ();
+		formatTag = USBToHostWord (thisStream->GetFormatTag ());
 	
 	return formatTag;
 }
@@ -484,10 +484,10 @@ UInt16 USBAudioConfigObject::GetIndexedInputTerminalType (UInt8 interfaceNum, UI
     USBAudioControlObject *				thisControl;
     UInt16								terminalType;
 
-	terminalType = INPUT_UNDEFINED;
+	terminalType = USBToHostWord (INPUT_UNDEFINED);
     thisControl = GetControlObject (interfaceNum, altInterfaceNum);
     if (thisControl)
-		terminalType = thisControl->GetIndexedInputTerminalType (index);
+		terminalType = USBToHostWord (thisControl->GetIndexedInputTerminalType (index));
 
     return terminalType;
 }
@@ -520,10 +520,10 @@ UInt16 USBAudioConfigObject::GetIndexedOutputTerminalType (UInt8 interfaceNum, U
     USBAudioControlObject *				thisControl;
     UInt16								terminalType;
 
-	terminalType = OUTPUT_UNDEFINED;
+	terminalType = USBToHostWord (OUTPUT_UNDEFINED);
     thisControl = GetControlObject (interfaceNum, altInterfaceNum);
     if (thisControl)
-		terminalType = thisControl->GetIndexedOutputTerminalType (index);
+		terminalType = USBToHostWord (thisControl->GetIndexedOutputTerminalType (index));
 
     return terminalType;
 }
@@ -532,10 +532,10 @@ UInt16 USBAudioConfigObject::GetInputTerminalType (UInt8 interfaceNum, UInt8 alt
     USBAudioControlObject *				thisControl;
     UInt16								terminalType;
 
-    terminalType = INPUT_UNDEFINED;
+    terminalType = USBToHostWord (INPUT_UNDEFINED);
     thisControl = GetControlObject (interfaceNum, altInterfaceNum);
     if (thisControl)
-		terminalType = thisControl->GetInputTerminalType (terminalNum);
+		terminalType = USBToHostWord (thisControl->GetInputTerminalType (terminalNum));
 
     return terminalType;
 }
@@ -595,7 +595,7 @@ UInt16 USBAudioConfigObject::GetMaxBitRate (UInt8 interfaceNum, UInt8 altInterfa
 	maxBitRate = 0;
 	thisStream = GetStreamObject (interfaceNum, altInterfaceNum);
 	if (thisStream)
-		maxBitRate = thisStream->GetMaxBitRate ();
+		maxBitRate = USBToHostWord (thisStream->GetMaxBitRate ());
 	
 	return maxBitRate;
 }
@@ -708,7 +708,7 @@ UInt16 USBAudioConfigObject::GetSamplesPerFrame (UInt8 interfaceNum, UInt8 altIn
 	samplesPerFrame = 0;
 	thisStream = GetStreamObject (interfaceNum, altInterfaceNum);
 	if (thisStream)
-		samplesPerFrame = thisStream->GetMaxBitRate ();
+		samplesPerFrame = USBToHostWord (thisStream->GetMaxBitRate ());
 	
 	return samplesPerFrame;
 }
@@ -777,10 +777,10 @@ UInt16 USBAudioConfigObject::GetOutputTerminalType (UInt8 interfaceNum, UInt8 al
     USBAudioControlObject *				thisControl;
     UInt16								terminalType;
 
-    terminalType = OUTPUT_UNDEFINED;
+    terminalType = USBToHostWord (OUTPUT_UNDEFINED);
     thisControl = GetControlObject (interfaceNum, altInterfaceNum);
     if (thisControl)
-		terminalType = thisControl->GetOutputTerminalType (terminalNum);
+		terminalType = USBToHostWord (thisControl->GetOutputTerminalType (terminalNum));
 
     return terminalType;
 }
@@ -1017,7 +1017,7 @@ USBInterfaceDescriptorPtr USBAudioConfigObject::ParseInterfaceDescriptor (USBInt
 		theControlInterfaceNum = ((ACInterfaceDescriptorPtr)theInterfacePtr)->bInterfaceNumber;
 
 		if (NULL == theControls) {
-			theControls = OSArray::withObjects (&(const OSObject *)theControlObject, 1);
+			theControls = OSArray::withObjects ((const OSObject **)&theControlObject, 1);
 			FailIf (NULL == theControls, Exit);
 		} else {
 			theControls->setObject (theControlObject);
@@ -1037,7 +1037,7 @@ USBInterfaceDescriptorPtr USBAudioConfigObject::ParseInterfaceDescriptor (USBInt
 		theStreamObject->SetInterfaceProtocol (theInterfacePtr->bInterfaceProtocol);
 
 		if (NULL == theStreams) {
-			theStreams = OSArray::withObjects (&(const OSObject *)theStreamObject, 1);
+			theStreams = OSArray::withObjects ((const OSObject **)&theStreamObject, 1);
 			FailIf (NULL == theStreams, Exit);
 		} else {
 			theStreams->setObject (theStreamObject);
@@ -1222,13 +1222,13 @@ UInt16 USBAudioControlObject::GetIndexedInputTerminalType (UInt8 index) {
     UInt16								terminalType;
 
 	thisInputTerminal = NULL;
-    terminalType = INPUT_UNDEFINED;
+    terminalType =  USBToHostWord (INPUT_UNDEFINED);
 	if (NULL != theInputTerminals) {
 		thisInputTerminal = OSDynamicCast (USBInputTerminalObject, theInputTerminals->getObject (index));
 	}
 
     if (thisInputTerminal)
-		terminalType = thisInputTerminal->GetTerminalType ();
+		terminalType = USBToHostWord (thisInputTerminal->GetTerminalType ());
 
 	return terminalType;
 }
@@ -1270,13 +1270,13 @@ UInt16 USBAudioControlObject::GetIndexedOutputTerminalType (UInt8 index) {
     UInt16								terminalType;
 
 	thisOutputTerminal = NULL;
-    terminalType = OUTPUT_UNDEFINED;
+    terminalType = USBToHostWord (OUTPUT_UNDEFINED);
 	if (NULL != theOutputTerminals) {
 		thisOutputTerminal = OSDynamicCast (USBOutputTerminalObject, theOutputTerminals->getObject (index));
 	}
 
     if (thisOutputTerminal)
-		terminalType = thisOutputTerminal->GetTerminalType ();
+		terminalType = USBToHostWord (thisOutputTerminal->GetTerminalType ());
 
 	return terminalType;
 }
@@ -1366,10 +1366,10 @@ UInt16 USBAudioControlObject::GetInputTerminalType (UInt8 unitID) {
     USBInputTerminalObject *			thisInputTerminal;
     UInt16								terminalType;
 
-    terminalType = INPUT_UNDEFINED;
+    terminalType = USBToHostWord (INPUT_UNDEFINED);
     thisInputTerminal = GetInputTerminalObject (unitID);
     if (thisInputTerminal)
-		terminalType = thisInputTerminal->GetTerminalType ();
+		terminalType = USBToHostWord (thisInputTerminal->GetTerminalType ());
 
     return terminalType;
 }
@@ -1484,10 +1484,10 @@ UInt16 USBAudioControlObject::GetOutputTerminalType (UInt8 unitID) {
     USBOutputTerminalObject *			thisOutputTerminal;
     UInt16								terminalType;
 
-    terminalType = OUTPUT_UNDEFINED;
+    terminalType = USBToHostWord (OUTPUT_UNDEFINED);
     thisOutputTerminal = GetOutputTerminalObject (unitID);
     if (thisOutputTerminal)
-		terminalType = thisOutputTerminal->GetTerminalType ();
+		terminalType = USBToHostWord (thisOutputTerminal->GetTerminalType ());
 
     return terminalType;
 }
@@ -1549,7 +1549,7 @@ USBInterfaceDescriptorPtr USBAudioControlObject::ParseACInterfaceDescriptor (USB
                 inputTerminal->SetChannelConfig (USBToHostWord (((ACInputTerminalDescriptorPtr)theInterfacePtr)->wChannelConfig));
 
 				if (NULL == theInputTerminals) {
-					theInputTerminals = OSArray::withObjects (&(const OSObject *)inputTerminal, 1);
+					theInputTerminals = OSArray::withObjects ((const OSObject **)&inputTerminal, 1);
 					FailIf (NULL == theInputTerminals, Exit);
 				} else {
 					theInputTerminals->setObject (inputTerminal);
@@ -1568,7 +1568,7 @@ USBInterfaceDescriptorPtr USBAudioControlObject::ParseACInterfaceDescriptor (USB
                 outputTerminal->SetSourceID (((ACOutputTerminalDescriptorPtr)theInterfacePtr)->bSourceID);
 
 				if (NULL == theOutputTerminals) {
-					theOutputTerminals = OSArray::withObjects (&(const OSObject *)outputTerminal, 1);
+					theOutputTerminals = OSArray::withObjects ((const OSObject **)&outputTerminal, 1);
 					FailIf (NULL == theOutputTerminals, Exit);
 				} else {
 					theOutputTerminals->setObject (outputTerminal);
@@ -1592,7 +1592,7 @@ USBInterfaceDescriptorPtr USBAudioControlObject::ParseACInterfaceDescriptor (USB
 				featureUnit->InitControlsArray (&((ACFeatureUnitDescriptorPtr)theInterfacePtr)->bmaControls[0], numControls);
 
 				if (NULL == theFeatureUnits) {
-					theFeatureUnits = OSArray::withObjects (&(const OSObject *)featureUnit, 1);
+					theFeatureUnits = OSArray::withObjects ((const OSObject **)&featureUnit, 1);
 					FailIf (NULL == theFeatureUnits, Exit);
 				} else {
 					theFeatureUnits->setObject (featureUnit);
@@ -1628,7 +1628,7 @@ USBInterfaceDescriptorPtr USBAudioControlObject::ParseACInterfaceDescriptor (USB
 				mixerUnit->InitControlsArray (&((ACMixerUnitDescriptorPtr)theInterfacePtr)->baSourceID[((ACMixerUnitDescriptorPtr)theInterfacePtr)->bNrInPins + 3], controlSize);
 
 				if (NULL == theMixerUnits) {
-					theMixerUnits = OSArray::withObjects (&(const OSObject *)mixerUnit, 1);
+					theMixerUnits = OSArray::withObjects ((const OSObject **)&mixerUnit, 1);
 					FailIf (NULL == theMixerUnits, Exit);
 				} else {
 					theMixerUnits->setObject (mixerUnit);
@@ -1647,7 +1647,7 @@ USBInterfaceDescriptorPtr USBAudioControlObject::ParseACInterfaceDescriptor (USB
 				debug2IOLog ("numInPins on selector = %d\n", ((ACSelectorUnitDescriptorPtr)theInterfacePtr)->bNrInPins);
 
 				if (NULL == theSelectorUnits) {
-					theSelectorUnits = OSArray::withObjects (&(const OSObject *)selectorUnit, 1);
+					theSelectorUnits = OSArray::withObjects ((const OSObject **)&selectorUnit, 1);
 					FailIf (NULL == theSelectorUnits, Exit);
 				} else {
 					theSelectorUnits->setObject (selectorUnit);
@@ -1799,6 +1799,12 @@ UInt8 USBAudioStreamObject::GetIsocEndpointSyncType (UInt8 address) {
 	return syncType;
 }
 
+inline UInt32 ConvertSampleFreq(UInt8 *p)	// convert little-endian 24-bit (unsigned) to native 32-bit.  Make it a macro.
+{
+	UInt32 bigEndianSampleFreq = (p[2] << 16) | (p[1] << 8) | p[0];
+	return (bigEndianSampleFreq);
+}
+
 USBInterfaceDescriptorPtr USBAudioStreamObject::ParseASInterfaceDescriptor (USBInterfaceDescriptorPtr theInterfacePtr, UInt8 const currentInterface) {
     UInt32								indx;
 	UInt16								wFormatTag;
@@ -1819,7 +1825,7 @@ USBInterfaceDescriptorPtr USBAudioStreamObject::ParseASInterfaceDescriptor (USBI
                     debugIOLog ("in AS_GENERAL in ParseASInterfaceDescriptor\n");
                     terminalLink = ((ASInterfaceDescriptorPtr)theInterfacePtr)->bTerminalLink;
                     delay = ((ASInterfaceDescriptorPtr)theInterfacePtr)->bDelay;
-                    formatTag = (((ASInterfaceDescriptorPtr)theInterfacePtr)->wFormatTag[1] << 8) | ((ASInterfaceDescriptorPtr)theInterfacePtr)->wFormatTag[0];
+                    formatTag = USBToHostWord ((((ASInterfaceDescriptorPtr)theInterfacePtr)->wFormatTag[1] << 8) | ((ASInterfaceDescriptorPtr)theInterfacePtr)->wFormatTag[0]);
                     theInterfacePtr = (USBInterfaceDescriptorPtr)((UInt8 *)theInterfacePtr + theInterfacePtr->bLength);
                     break;
                 case FORMAT_TYPE:
@@ -1836,19 +1842,21 @@ USBInterfaceDescriptorPtr USBAudioStreamObject::ParseASInterfaceDescriptor (USBI
 								debugIOLog ("device has a discrete number of sample rates\n");
 								sampleFreqs = (UInt32 *)IOMalloc (numSampleFreqs * sizeof (UInt32));
 								for (indx = 0; indx < numSampleFreqs; indx++) {
-									((UInt8 *)sampleFreqs)[indx * 4] = 0;
+									sampleFreqs[indx] = ConvertSampleFreq( &((ASFormatTypeIDescriptorPtr)theInterfacePtr)->sampleFreq[indx * 3] );
+									/*((UInt8 *)sampleFreqs)[indx * 4] = 0;
 									((UInt8 *)sampleFreqs)[indx * 4 + 1] = ((ASFormatTypeIDescriptorPtr)theInterfacePtr)->sampleFreq[indx * 3 + 2];
 									((UInt8 *)sampleFreqs)[indx * 4 + 2] = ((ASFormatTypeIDescriptorPtr)theInterfacePtr)->sampleFreq[indx * 3 + 1];
-									((UInt8 *)sampleFreqs)[indx * 4 + 3] = ((ASFormatTypeIDescriptorPtr)theInterfacePtr)->sampleFreq[indx * 3];
+									((UInt8 *)sampleFreqs)[indx * 4 + 3] = ((ASFormatTypeIDescriptorPtr)theInterfacePtr)->sampleFreq[indx * 3]; */
 								}
 							} else {
 								debugIOLog ("device has a variable number of sample rates\n");
 								sampleFreqs = (UInt32 *)IOMalloc (2 * sizeof (UInt32));
 								for (indx = 0; indx < 2; indx++) {
-									((UInt8 *)sampleFreqs)[indx * 4] = 0;
+									sampleFreqs[indx] = ConvertSampleFreq( &((ASFormatTypeIDescriptorPtr)theInterfacePtr)->sampleFreq[indx * 3] );
+									/*((UInt8 *)sampleFreqs)[indx * 4] = 0;
 									((UInt8 *)sampleFreqs)[indx * 4 + 1] = ((ASFormatTypeIDescriptorPtr)theInterfacePtr)->sampleFreq[indx * 3 + 2];
 									((UInt8 *)sampleFreqs)[indx * 4 + 2] = ((ASFormatTypeIDescriptorPtr)theInterfacePtr)->sampleFreq[indx * 3 + 1];
-									((UInt8 *)sampleFreqs)[indx * 4 + 3] = ((ASFormatTypeIDescriptorPtr)theInterfacePtr)->sampleFreq[indx * 3];
+									((UInt8 *)sampleFreqs)[indx * 4 + 3] = ((ASFormatTypeIDescriptorPtr)theInterfacePtr)->sampleFreq[indx * 3]; */
 								}
 							}
 							break;
@@ -1861,19 +1869,21 @@ USBInterfaceDescriptorPtr USBAudioStreamObject::ParseASInterfaceDescriptor (USBI
 								debugIOLog ("device has a discrete number of sample rates\n");
 								sampleFreqs = (UInt32 *)IOMalloc (numSampleFreqs * sizeof (UInt32));
 								for (indx = 0; indx < numSampleFreqs; indx++) {
-									((UInt8 *)sampleFreqs)[indx * 4] = 0;
+									sampleFreqs[indx] = ConvertSampleFreq( &((ASFormatTypeIIDescriptorPtr)theInterfacePtr)->sampleFreq[indx * 3] );
+									/*((UInt8 *)sampleFreqs)[indx * 4] = 0;
 									((UInt8 *)sampleFreqs)[indx * 4 + 1] = ((ASFormatTypeIIDescriptorPtr)theInterfacePtr)->sampleFreq[indx * 3 + 2];
 									((UInt8 *)sampleFreqs)[indx * 4 + 2] = ((ASFormatTypeIIDescriptorPtr)theInterfacePtr)->sampleFreq[indx * 3 + 1];
-									((UInt8 *)sampleFreqs)[indx * 4 + 3] = ((ASFormatTypeIIDescriptorPtr)theInterfacePtr)->sampleFreq[indx * 3];
+									((UInt8 *)sampleFreqs)[indx * 4 + 3] = ((ASFormatTypeIIDescriptorPtr)theInterfacePtr)->sampleFreq[indx * 3]; */
 								}
 							} else {
 								debugIOLog ("device has a variable number of sample rates\n");
 								sampleFreqs = (UInt32 *)IOMalloc (2 * sizeof (UInt32));
 								for (indx = 0; indx < 2; indx++) {
-									((UInt8 *)sampleFreqs)[indx * 4] = 0;
+									sampleFreqs[indx] = ConvertSampleFreq( &((ASFormatTypeIIDescriptorPtr)theInterfacePtr)->sampleFreq[indx * 3] );
+									/*((UInt8 *)sampleFreqs)[indx * 4] = 0;
 									((UInt8 *)sampleFreqs)[indx * 4 + 1] = ((ASFormatTypeIIDescriptorPtr)theInterfacePtr)->sampleFreq[indx * 3 + 2];
 									((UInt8 *)sampleFreqs)[indx * 4 + 2] = ((ASFormatTypeIIDescriptorPtr)theInterfacePtr)->sampleFreq[indx * 3 + 1];
-									((UInt8 *)sampleFreqs)[indx * 4 + 3] = ((ASFormatTypeIIDescriptorPtr)theInterfacePtr)->sampleFreq[indx * 3];
+									((UInt8 *)sampleFreqs)[indx * 4 + 3] = ((ASFormatTypeIIDescriptorPtr)theInterfacePtr)->sampleFreq[indx * 3]; */
 								}
 							}
 							break;
@@ -1884,23 +1894,23 @@ USBInterfaceDescriptorPtr USBAudioStreamObject::ParseASInterfaceDescriptor (USBI
                     break;
 				case FORMAT_SPECIFIC:
                     debugIOLog ("in FORMAT_SPECIFIC in ParseASInterfaceDescriptor\n");
-					wFormatTag = ((ASFormatSpecificDescriptorHeaderPtr)theInterfacePtr)->wFormatTag[1] << 8 | ((ASFormatSpecificDescriptorHeaderPtr)theInterfacePtr)->wFormatTag[0];
+					wFormatTag = USBToHostWord (((ASFormatSpecificDescriptorHeaderPtr)theInterfacePtr)->wFormatTag[1] << 8 | ((ASFormatSpecificDescriptorHeaderPtr)theInterfacePtr)->wFormatTag[0]);
 					switch (wFormatTag) {
 						case MPEG:
 							debugIOLog ("in FORMAT_SPECIFIC in MPEG\n");
-							bmMPEGCapabilities = ((ASMPEGFormatSpecificDescriptorPtr)theInterfacePtr)->bmMPEGCapabilities[1] << 8 |
-												 ((ASMPEGFormatSpecificDescriptorPtr)theInterfacePtr)->bmMPEGCapabilities[0];
+							bmMPEGCapabilities = USBToHostWord(
+												((ASMPEGFormatSpecificDescriptorPtr)theInterfacePtr)->bmMPEGCapabilities[1] << 8 |
+												((ASMPEGFormatSpecificDescriptorPtr)theInterfacePtr)->bmMPEGCapabilities[0]);
 							bmMPEGFeatures = ((ASMPEGFormatSpecificDescriptorPtr)theInterfacePtr)->bmMPEGFeatures;
 							break;
 						case AC3:
 							debugIOLog ("in FORMAT_SPECIFIC in AC3\n");
-							bmAC3BSID = ((ASAC3FormatSpecificDescriptorPtr)theInterfacePtr)->bmBSID[3] << 24 |
+							bmAC3BSID = USBToHostLong(
+										((ASAC3FormatSpecificDescriptorPtr)theInterfacePtr)->bmBSID[3] << 24 |
 										((ASAC3FormatSpecificDescriptorPtr)theInterfacePtr)->bmBSID[2] << 16 |
 										((ASAC3FormatSpecificDescriptorPtr)theInterfacePtr)->bmBSID[1] << 8 |
-										((ASAC3FormatSpecificDescriptorPtr)theInterfacePtr)->bmBSID[0];
-							IOLog ("bmAC3BSID = 0x%x\n", bmAC3BSID);
+										((ASAC3FormatSpecificDescriptorPtr)theInterfacePtr)->bmBSID[0]);
 							bmAC3Features = ((ASAC3FormatSpecificDescriptorPtr)theInterfacePtr)->bmAC3Features;
-							IOLog ("bmAC3Features = 0x%x\n", bmAC3Features);
 							break;
 						default:
 							debug2IOLog ("!!!!unknown format type 0x%x in FORMAT_SPECIFIC!!!!\n", wFormatTag);
@@ -1929,7 +1939,7 @@ USBInterfaceDescriptorPtr USBAudioStreamObject::ParseASInterfaceDescriptor (USBI
 					theEndpoint->SetSynchAddress (((USBEndpointDescriptorPtr)theInterfacePtr)->bSynchAddress);
 
 					if (NULL == theEndpointObjects) {
-						theEndpointObjects = OSArray::withObjects (&(const OSObject *)theEndpoint, 1);
+						theEndpointObjects = OSArray::withObjects ((const OSObject **)&theEndpoint, 1);
 						FailIf (NULL == theEndpointObjects, Exit);
 					} else {
 						theEndpointObjects->setObject (theEndpoint);
@@ -1946,7 +1956,7 @@ USBInterfaceDescriptorPtr USBAudioStreamObject::ParseASInterfaceDescriptor (USBI
                                                                             ((ASEndpointDescriptorPtr)theInterfacePtr)->bmAttributes & (1 << pitchControlBit),
                                                                             ((ASEndpointDescriptorPtr)theInterfacePtr)->bmAttributes & (1 << maxPacketsOnlyBit),
                                                                             ((ASEndpointDescriptorPtr)theInterfacePtr)->bLockDelayUnits,
-                                                                            (((ASEndpointDescriptorPtr)theInterfacePtr)->wLockDelay[1] << 8) | ((ASEndpointDescriptorPtr)theInterfacePtr)->wLockDelay[0]);
+																			USBToHostWord (((UInt16 *)((ASEndpointDescriptorPtr)theInterfacePtr)->wLockDelay)[0]));
                     }
                     theInterfacePtr = (USBInterfaceDescriptorPtr)((UInt8 *)theInterfacePtr + theInterfacePtr->bLength);
                     break;
